#include "midi.h"
#include <avr/interrupt.h>
#include <avr/io.h>
/* ****************************** */
/* Midi uart for ATMEGA8 and 16 * */
/* ****************************** */

#ifndef F_CPU
/* Warning if F_CPU isnt defined in AVR Studio Options or in makefile */
#warning "F_CPU F_CPU not defined, using default 1MHz"
#define F_CPU 1000000UL  // F_CPU in Hz - definition as unsigned long 
#endif
 
#define BAUD 31250UL      // std midi baud rate
#include <util/setbaud.h>

/* Midi uart teste only for program change events, for guitar footcontrols */
void initMidiUart(unsigned char receiveOnly )   
{   	
	//init global variables
	UART_RxTail = 0;
	UART_RxHead = 0;
	UART_TxTail = 0;
	UART_TxHead = 0;         

   	UBRRH = UBRRH_VALUE;
   	UBRRL = UBRRL_VALUE;
   /* see -> http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial/Der_UART  */
#if USE_2X // U2X-Modus 
   	UCSRA |= (1 << U2X);
#else   // Non U2X-Modus -> setbaud.h
   UCSRA &= ~(1 << U2X);
#endif

	enableMIDI(receiveOnly );
	
	UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0); // Asynchron 8N1          
	
}
/* not testet so far */
void enableMIDI(unsigned char receiveOnly )
{
	if ( (UCSRB & (1 << RXCIE)) ) return;//already enabled...I'm stubborn and paronid
	
	if (receiveOnly) UCSRB = (( 1 << RXCIE ) | ( 1 << RXEN ) ); 
	else UCSRB = (( 1 << RXCIE ) | ( 1 << RXEN ) | ( 1 << TXEN )); 

}
/* not testet so far */
void disableMIDI()
{	
	if ( !(UCSRB & (1 << RXCIE)) ) return; //already disabled

	UCSRB &= ~( ( 1 << RXCIE ) |  (1<<RXEN) | (1<<RXCIE) | ( 1 << TXEN ));

}
unsigned char readByte( void )
{
	unsigned char newTail;
	unsigned char byte ;
 
    newTail = (UART_RxTail + 1) & UART_RX_BUFFER_MASK;
    
	byte = UART_RxBuf[newTail];
	UART_RxTail = newTail; 
 
    return byte;
}
void writeByte( unsigned char byte )
{
	if ( !(UCSRB & (1 << RXCIE)) ) return; //avoid hang if transmit is disabled
	unsigned char newHead; 
 
    newHead  = (UART_TxHead + 1) & UART_TX_BUFFER_MASK;
 
    while ( newHead == UART_TxTail ) {
        ;// wait for free space in buffer 
    }
 
    UART_TxBuf[newHead] = byte;
    UART_TxHead = newHead;

	UCSRB |= (1<<UDRIE); // Disable IRQ for ATMEGA8 and 16    
} 

void writeString(const char *string )
{
    while (*string) writeByte(*string++); 
}

unsigned char bytesAvailable( void )
{
    // Return 0 (FALSE) if the receive buffer is empty 
	return ( UART_RxHead != UART_RxTail ); 
}
// Receiver IRQ ATMega8 and 16
ISR (USART_RXC_vect) //mega168 USART_RX_vect -> http://www.nongnu.org/avr-libc/user-manual/group__avr__interrupts.html   
{
	unsigned char byte;
	unsigned char newHead;
 
	byte = UDR;//read received byte
 
	/* Calculate buffer index */
	newHead = ( UART_RxHead + 1 ) & UART_RX_BUFFER_MASK;
 
	if ( newHead == UART_RxTail ) {
		//unhandled buffer overflow error
	}
	else {
        UART_RxHead = newHead; //new head
        UART_RxBuf[newHead] = byte; //append the read byte
    } 
}
// Sender IRQ ATMega8 and 16
ISR (USART_UDRE_vect) //also mega168, SIG_UART_DATA -> http://www.nongnu.org/avr-libc/user-manual/group__avr__interrupts.html
{
	unsigned char newTail;
 
	/* Check if all data is transmitted */
	if ( UART_TxHead != UART_TxTail ){
		newTail = ( UART_TxTail + 1 ) & UART_TX_BUFFER_MASK;// new Tail
		UART_TxTail = newTail;      /* Store new index */
 
		UDR = UART_TxBuf[newTail];  //send the byte
	}
	else {      		
		UCSRB &= ~(1<<UDRIE);// Enable IRQ for ATMEGA8 and 16          
	}
}
